@@ -3,7 +3,9 @@ from PySide2.QtGui import QColor
 class Setup:
     DB_PATH = "jal.sqlite"
     INIT_SCRIPT_PATH = 'jal_init.sql'
-    TARGET_SCHEMA = 12
+    UPDATES_PATH = 'updates'
+    UPDATE_PREFIX = 'jal_delta_'
+    TARGET_SCHEMA = 13
     CALC_TOLERANCE = 1e-10
     DISP_TOLERANCE = 1e-4
 
