@@ -1,6 +1,6 @@
 import logging
 import xml.etree.ElementTree as xml_tree
-from datetime import datetime
+from datetime import datetime, timedelta
 from io import StringIO
 
 import pandas as pd
@@ -117,7 +117,7 @@ class QuoteDownloader(QObject):
                 logging.warning(g_tr('QuotesUpdateDialog', "No data were downloaded for ") + f"{asset}")
                 continue
             if data is not None:
-                for date, quote in data.iterrows():
+                for date, quote in data.iterrows():  # Date in pandas dataset is in UTC by default
                     self.SubmitQuote(asset_id, asset, int(date.timestamp()), float(quote[0]))
         logging.info(g_tr('QuotesUpdateDialog', "Download completed"))
 
@@ -138,8 +138,8 @@ class QuoteDownloader(QObject):
         return None
 
     def CBR_DataReader(self, currency_code, _isin, start_timestamp, end_timestamp):
-        date1 = datetime.fromtimestamp(start_timestamp).strftime('%d/%m/%Y')
-        date2 = datetime.fromtimestamp(end_timestamp + 86400).strftime('%d/%m/%Y')  # add 1 as CBR sets rate a day ahead
+        date1 = datetime.utcfromtimestamp(start_timestamp).strftime('%d/%m/%Y')
+        date2 = (datetime.utcfromtimestamp(end_timestamp) + timedelta(days=1)).strftime('%d/%m/%Y')  # add 1 as CBR sets rate a day ahead
         code = str(self.CBR_codes.loc[self.CBR_codes["ISO_name"] == currency_code, "CBR_code"].values[0]).strip()
         url = f"http://www.cbr.ru/scripts/XML_dynamic.asp?date_req1={date1}&date_req2={date2}&VAL_NM_RQ={code}"
         xml_root = xml_tree.fromstring(get_web_data(url))
@@ -178,8 +178,8 @@ class QuoteDownloader(QObject):
             logging.warning(f"Failed to find {asset_code} at {url}")
             return None
 
-        date1 = datetime.fromtimestamp(start_timestamp).strftime('%Y-%m-%d')
-        date2 = datetime.fromtimestamp(end_timestamp).strftime('%Y-%m-%d')
+        date1 = datetime.utcfromtimestamp(start_timestamp).strftime('%Y-%m-%d')
+        date2 = datetime.utcfromtimestamp(end_timestamp).strftime('%Y-%m-%d')
         url = f"http://iss.moex.com/iss/history/engines/"\
               f"{engine}/markets/{market}/boards/{board_id}/securities/{asset_code}.xml?from={date1}&till={date2}"
         xml_root = xml_tree.fromstring(get_web_data(url))
@@ -245,4 +245,4 @@ class QuoteDownloader(QObject):
                        [(":timestamp", timestamp), (":asset_id", asset_id), (":quote", quote)])
         self.db.commit()
         logging.info(g_tr('QuotesUpdateDialog', "Quote loaded: ") +
-                     f"{asset_name} @ {datetime.fromtimestamp(timestamp).strftime('%d/%m/%Y %H:%M:%S')} = {quote}")
+                     f"{asset_name} @ {datetime.utcfromtimestamp(timestamp).strftime('%d/%m/%Y %H:%M:%S')} = {quote}")
