@@ -1,7 +1,7 @@
 import logging
 import math
 import re
-from datetime import datetime
+from datetime import datetime, timezone
 
 import pandas
 from lxml import etree
@@ -9,7 +9,7 @@ from PySide2.QtCore import QObject, Signal, Slot
 from PySide2.QtSql import QSqlTableModel
 from PySide2.QtWidgets import QDialog, QFileDialog, QMessageBox
 from jal.constants import Setup, TransactionType, PredefinedAsset, PredefinedCategory, CorporateAction
-from jal.db.helpers import executeSQL, readSQL, get_country_by_code
+from jal.db.helpers import executeSQL, readSQL, get_country_by_code, account_last_date
 from jal.ui_custom.helpers import g_tr
 from jal.ui.ui_add_asset_dlg import Ui_AddAssetDialog
 from jal.ui.ui_select_account_dlg import Ui_SelectAccountDlg
@@ -35,6 +35,7 @@ class IBKR:
     DummyExchange = "VALUE"
     SpinOffPattern = "^(.*)\(.* SPINOFF +(\d+) +FOR +(\d+) +\(.*$"
     SplitPattern = "^.* SPLIT +(\d+) +FOR +(\d+) +\(.*$"
+    StockDividendPattern = "^.* STOCK DIVIDEND .* (\d+) +FOR +(\d+) +\(.*$"
 
     AssetType = {
         'CASH': PredefinedAsset.Money,
@@ -75,9 +76,9 @@ class IBKR:
         time_str = data.attrib[name]
         try:
             if len(time_str) == 15:  # YYYYMMDD;HHMMSS
-                return int(datetime.strptime(time_str, "%Y%m%d;%H%M%S").timestamp())
+                return int(datetime.strptime(time_str, "%Y%m%d;%H%M%S").replace(tzinfo=timezone.utc).timestamp())
             elif len(time_str) == 8: # YYYYMMDD
-                return int(datetime.strptime(time_str, "%Y%m%d").timestamp())
+                return int(datetime.strptime(time_str, "%Y%m%d").replace(tzinfo=timezone.utc).timestamp())
             else:
                 return default_value
         except ValueError:
@@ -379,6 +380,14 @@ class StatementLoader(QObject):
             for FlexStatements in xml_root.getroot():
                 for statement in FlexStatements:
                     attr = statement.attrib
+                    report_start = int(
+                        datetime.strptime(attr['fromDate'], "%Y%m%d").replace(tzinfo=timezone.utc).timestamp())
+                    if report_start < account_last_date(self.db, attr['accountId']):
+                        if QMessageBox().warning(None,
+                                                 g_tr('StatementLoader', "Confirmation"),
+                                                 g_tr('StatementLoader', "Statement period starts before last recorded operation for the account. Continue import?"),
+                                                 QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
+                            return False
                     logging.info(g_tr('StatementLoader', "Load IB Flex-statement for account ") +
                                  f"{attr['accountId']}: {attr['fromDate']} - {attr['toDate']}")
                     for section in section_loaders:
@@ -505,7 +514,7 @@ class StatementLoader(QObject):
                     logging.error(g_tr('StatementLoader', "Can't find paired record for ") + f"{action}")
                     continue
                 self.createCorpAction(action['accountId'], CorporateAction.Merger, action['dateTime'], action['transactionID'], paired_records[0]['symbol'],
-                                      -paired_records[0]['quantity'], action['symbol'], action['quantity'], action['description'])
+                                      -paired_records[0]['quantity'], action['symbol'], action['quantity'], 1, action['description'])
                 cnt += 2
             elif action['type'] == CorporateAction.SpinOff:
                 parts = re.match(IBKR.SpinOffPattern, action['description'], re.IGNORECASE)
@@ -517,7 +526,7 @@ class StatementLoader(QObject):
                 mult_b = int(parts.group(3))
                 qty_old = mult_b * action['quantity'] / mult_a
                 self.createCorpAction(action['accountId'], CorporateAction.SpinOff, action['dateTime'], action['transactionID'], asset_id_old,
-                                      qty_old, action['symbol'], action['quantity'], action['description'])
+                                      qty_old, action['symbol'], action['quantity'], 0, action['description'])
                 cnt += 1
             elif action['type'] == CorporateAction.SymbolChange:
                 # additional info is in next dummy record where old symbol is changed to *.OLD
@@ -532,22 +541,32 @@ class StatementLoader(QObject):
                     logging.error(g_tr('StatementLoader', "Can't find paired record for: ") + f"{action}")
                     continue
                 self.createCorpAction(action['accountId'], CorporateAction.SymbolChange, action['dateTime'], action['transactionID'], paired_records[0]['symbol'],
-                                      action['quantity'], action['symbol'], action['quantity'], action['description'])
+                                      action['quantity'], action['symbol'], action['quantity'], 1, action['description'])
                 cnt += 2
             elif action['type'] == CorporateAction.StockDividend:
-                self.createCorpAction(action['accountId'], CorporateAction.StockDividend, action['dateTime'], action['transactionID'], action['symbol'], 0,
-                                      action['symbol'], action['quantity'], action['description'])
+                parts = re.match(IBKR.StockDividendPattern, action['description'], re.IGNORECASE)
+                if not parts:
+                    logging.error(
+                        g_tr('StatementLoader', "Failed to parse corp.action Stock Dividend data for ") + f"{action}")
+                    return
+                mult_a = int(parts.group(1))
+                mult_b = int(parts.group(2))
+                qty_old = mult_b * action['quantity'] / mult_a
+                if abs(qty_old - round(qty_old)) < 0.00001:   # We got an approximation - take closest integer if it's close enough
+                    qty_old = round(qty_old)
+                self.createCorpAction(action['accountId'], CorporateAction.StockDividend, action['dateTime'], action['transactionID'], action['symbol'], qty_old,
+                                      action['symbol'], action['quantity'], 1, action['description'])
                 cnt += 1
             elif action['type'] == CorporateAction.Split:
                 parts = re.match(IBKR.SplitPattern, action['description'], re.IGNORECASE)
                 if not parts:
-                    logging.error(g_tr('StatementLoader', "Failed to parse corp.action Split data"))
+                    logging.error(g_tr('StatementLoader', "Failed to parse corp.action Split data for ") + f"{action}")
                     return
                 mult_a = int(parts.group(1))
                 mult_b = int(parts.group(2))
                 qty_new = mult_a * action['quantity'] / mult_b
                 self.createCorpAction(action['accountId'], CorporateAction.Split, action['dateTime'], action['transactionID'], action['symbol'],
-                                      action['quantity'], action['symbol'], qty_new, action['description'])
+                                      action['quantity'], action['symbol'], qty_new, 1, action['description'])
                 cnt += 1
             else:
                 logging.error(g_tr('StatementLoader', "Corporate action type is not supported: ")
@@ -688,7 +707,8 @@ class StatementLoader(QObject):
                             (":f_amount", f_amount), (":t_amount", t_amount), (":note", note)])
         self.db.commit()
 
-    def createCorpAction(self, account_id, type, timestamp, number, asset_id_old, qty_old, asset_id_new, qty_new, note):
+    def createCorpAction(self, account_id, type, timestamp, number, asset_id_old, qty_old, asset_id_new, qty_new,
+                         basis_ratio, note):
         action_id = readSQL(self.db,
                            "SELECT id FROM corp_actions "
                            "WHERE timestamp=:timestamp AND type = :type AND account_id = :account AND number = :number "
@@ -701,11 +721,12 @@ class StatementLoader(QObject):
 
         _ = executeSQL(self.db,
                        "INSERT INTO corp_actions (timestamp, number, account_id, type, "
-                       "asset_id, qty, asset_id_new, qty_new, note) "
-                       "VALUES (:timestamp, :number, :account, :type, :asset, :qty, :asset_new, :qty_new, :note)",
+                       "asset_id, qty, asset_id_new, qty_new, basis_ratio, note) "
+                       "VALUES (:timestamp, :number, :account, :type, "
+                       ":asset, :qty, :asset_new, :qty_new, :basis_ratio, :note)",
                        [(":timestamp", timestamp), (":number", number), (":account", account_id), (":type", type),
-                        (":asset", asset_id_old), (":qty", float(qty_old)),
-                        (":asset_new", asset_id_new), (":qty_new", float(qty_new)), (":note", note)])
+                        (":asset", asset_id_old), (":qty", float(qty_old)), (":asset_new", asset_id_new),
+                        (":qty_new", float(qty_new)), (":basis_ratio", basis_ratio), (":note", note)])
         self.db.commit()
 
     def loadIBDividend(self, dividend):
@@ -747,11 +768,11 @@ class StatementLoader(QObject):
     def loadIBDepositWithdraw(self, cash):
         if cash['amount'] >= 0:     # Deposit
             text = g_tr('StatementLoader', "Deposit of ") + f"{cash['amount']:.2f} {cash['currency']} " + \
-                   f"@{datetime.fromtimestamp(cash['dateTime']).strftime('%d.%m.%Y')}\n" + \
+                   f"@{datetime.utcfromtimestamp(cash['dateTime']).strftime('%d.%m.%Y')}\n" + \
                    g_tr('StatementLoader', "Select account to withdraw from:")
         else:                       # Withdrawal
             text = g_tr('StatementLoader', "Withdrawal of ") + f"{-cash['amount']:.2f} {cash['currency']} " + \
-                   f"@{datetime.fromtimestamp(cash['dateTime']).strftime('%d.%m.%Y')}\n" + \
+                   f"@{datetime.utcfromtimestamp(cash['dateTime']).strftime('%d.%m.%Y')}\n" + \
                    g_tr('StatementLoader', "Select account to deposit to:")
 
         dialog = SelectAccountDialog(self.parent, self.db, text, cash['accountId'])
@@ -844,8 +865,8 @@ class StatementLoader(QObject):
             if asset_id is None:
                 logging.warning(g_tr('StatementLoader', "Unknown asset ") + f"'{row[Quik.Symbol]}'")
                 continue
-            timestamp = int(datetime.strptime(row[Quik.DateTime], "%d.%m.%Y %H:%M:%S").timestamp())
-            settlement = int(datetime.strptime(row[Quik.SettleDate], "%d.%m.%Y").timestamp())
+            timestamp = int(datetime.strptime(row[Quik.DateTime], "%d.%m.%Y %H:%M:%S").replace(tzinfo=timezone.utc).timestamp())
+            settlement = int(datetime.strptime(row[Quik.SettleDate], "%d.%m.%Y").replace(tzinfo=timezone.utc).timestamp())
             number = row[Quik.TradeNumber]
             price = row[Quik.Price]
             amount = row[Quik.Amount]
