@@ -19,7 +19,7 @@ class RebuildDialog(QDialog, Ui_ReBuildDialog):
 
         self.LastRadioButton.toggle()
         self.frontier = frontier
-        frontier_text = datetime.fromtimestamp(frontier).strftime('%d/%m/%Y')
+        frontier_text = datetime.utcfromtimestamp(frontier).strftime('%d/%m/%Y')
         self.FrontierDateLabel.setText(frontier_text)
         self.CustomDateEdit.setDate(QDate.currentDate())
 
@@ -276,31 +276,63 @@ class Ledger:
         # Get asset amount accumulated before current operation
         asset_amount = self.getAmount(BookAccount.Assets, asset_id)
         if ((-type) * asset_amount) > 0:  # Process deal match if we have asset that is opposite to operation
-            # Get information about last deal with quantity of opposite sign
-            last_sid = readSQL(self.db, "SELECT "
-                                        "CASE WHEN (:type)*qty<0 THEN open_sid ELSE close_sid END AS last_sid "
-                                        "FROM deals "
-                                        "WHERE account_id=:account_id AND asset_id=:asset_id "
-                                        "ORDER BY close_sid DESC, open_sid DESC LIMIT 1",
-                               [(":type", type), (":account_id", account_id), (":asset_id", asset_id)])
-            last_sid = 0 if last_sid is None else last_sid
-            # Next get information about abs trade quantity that was in this last deal
-            last_qty = readSQL(self.db, "SELECT coalesce(ABS(t.qty), 0)+coalesce(ABS(ca.qty_new) , 0) AS qty "
-                                        "FROM sequence AS s "
-                                        "LEFT JOIN trades AS t ON t.id=s.operation_id AND s.type=3 "
-                                        "LEFT JOIN corp_actions AS ca ON ca.id=s.operation_id AND s.type=5 "
-                                        "WHERE s.id=:last_sid", [(":last_sid", last_sid)])
-            last_qty = 0 if last_qty is None else last_qty
-            # Collect quantity of all deals where this last opposite trade participated (positive value)
-            deals_qty = readSQL(self.db, "SELECT coalesce(SUM(ABS(qty)), 0) "
-                                         "FROM deals AS d "
-                                         "WHERE account_id=:account_id AND asset_id=:asset_id "
-                                         "AND (open_sid=:last_sid OR close_sid=:last_sid)",
-                                [(":account_id", account_id), (":asset_id", asset_id), (":last_sid", last_sid)])
-            reminder = last_qty - deals_qty
-            # if last trade is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
-            if reminder > 0:
-                last_sid = last_sid - 1
+            last_type = readSQL(self.db, "SELECT s.type FROM deals AS d "
+                                         "LEFT JOIN sequence AS s ON d.close_sid=s.id "
+                                         "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
+                                         "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
+                                [(":account_id", account_id), (":asset_id", asset_id)])
+            if last_type is None or last_type == TransactionType.Trade:
+                # Get information about last deal with quantity of opposite sign
+                last_sid = readSQL(self.db, "SELECT "
+                                            "CASE WHEN (:type)*qty<0 THEN open_sid ELSE close_sid END AS last_sid "
+                                            "FROM deals "
+                                            "WHERE account_id=:account_id AND asset_id=:asset_id "
+                                            "ORDER BY close_sid DESC, open_sid DESC LIMIT 1",
+                                   [(":type", type), (":account_id", account_id), (":asset_id", asset_id)])
+                last_sid = 0 if last_sid is None else last_sid
+                # Next get information about abs trade quantity that was in this last deal
+                # It may be a corporate action - its quantity calculation is a bit more complicated
+                last_qty = readSQL(self.db,
+                                              "SELECT coalesce(SUM(qty), 0) AS qty FROM ( "
+                                              "SELECT ABS(t.qty) AS qty "
+                                              "FROM sequence AS s "
+                                              "LEFT JOIN trades AS t ON t.id=s.operation_id AND s.type=3 "
+                                              "WHERE s.id=:last_sid "
+                                              "UNION ALL "
+                                              "SELECT "
+                                              "CASE "
+                                              "    WHEN ca.type = 2 AND ca.asset_id=:asset_id THEN ca.qty "
+                                              "    ELSE ca.qty_new "
+                                              "END AS qty "
+                                              "FROM sequence AS s "
+                                              "LEFT JOIN corp_actions AS ca ON ca.id=s.operation_id AND s.type=5 "
+                                              "WHERE s.id=:last_sid "
+                                              ")",
+                                              [(":asset_id", asset_id), (":last_sid", last_sid)])
+                # Collect quantity of all deals where this last opposite trade participated (positive value)
+                # If it was a corporate action we need to take only where it was an opening of the deal
+                deals_qty = readSQL(self.db, "SELECT coalesce(SUM(ABS(qty)), 0) "   
+                                             "FROM deals AS d "
+                                             "LEFT JOIN sequence AS s ON s.id=d.close_sid "
+                                             "WHERE account_id=:account_id AND asset_id=:asset_id "
+                                             "AND (open_sid=:last_sid OR close_sid=:last_sid) AND s.type!=5",
+                                    [(":account_id", account_id), (":asset_id", asset_id), (":last_sid", last_sid)])
+                reminder = last_qty - deals_qty
+                # if last trade is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
+                if reminder > 0:
+                    last_sid -= 1
+            elif last_type == TransactionType.CorporateAction:
+                last_sid, ca_type = readSQL(self.db,
+                                            "SELECT d.close_sid, c.type FROM deals AS d "
+                                            "LEFT JOIN sequence AS s ON d.close_sid=s.id "
+                                            "LEFT JOIN corp_actions AS c ON s.operation_id=c.id "
+                                            "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
+                                            "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
+                                            [(":account_id", account_id), (":asset_id", asset_id)])
+                if ca_type == CorporateAction.Split \
+                        or ca_type == CorporateAction.StockDividend or ca_type == CorporateAction.SpinOff:
+                    last_sid -= 1
+                reminder = 0
 
             # Get a list of all previous not matched trades or corporate actions of opposite direction (type parameter)
             query = executeSQL(self.db,
@@ -310,10 +342,19 @@ class Ledger:
                                "WHERE (:type)*qty < 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
                                "AND s.id < :sid AND s.id > :last_sid "
                                "UNION ALL "
-                               "SELECT s.id, ABS(c.qty_new), coalesce(l.value/c.qty_new, 0) AS price FROM corp_actions AS c "
+                               "SELECT s.id, "
+                               "CASE "
+                               "    WHEN c.type = 2 AND c.asset_id=:asset_id THEN c.qty "
+                               "    ELSE c.qty_new "
+                               "END AS qty, "
+                               "CASE "
+                               "    WHEN c.type = 2 AND c.asset_id=:asset_id THEN coalesce(l.value/c.qty, 0) "
+                               "    ELSE coalesce(l.value/c.qty_new, 0) "
+                               "END AS price "
+                               "FROM corp_actions AS c "
                                "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
-                               "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id_new AND l.value > 0 "
-                               "WHERE (:type)*c.qty_new < 0 AND c.asset_id_new=:asset_id AND c.account_id=:account_id "
+                               "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=:asset_id AND l.value > 0 "
+                               "WHERE (:type)*c.qty_new < 0 AND (c.asset_id_new=:asset_id OR (c.asset_id=:asset_id AND c.type=2)) AND c.account_id=:account_id "
                                "AND s.id < :sid AND s.id > :last_sid "
                                ")ORDER BY id",
                                [(":type", type), (":asset_id", asset_id), (":account_id", account_id),
@@ -388,96 +429,103 @@ class Ledger:
         }
         operationTransfer[self.current['subtype']]()
 
-    def processAssetConversion(self):
+    def processCorporateAction(self):
         seq_id = self.current_seq
         account_id = self.current['account']
         asset_id = self.current['asset']
         qty = self.current['amount']
-        buy_qty = 0
-        buy_value = 0
-        if self.getAmount(BookAccount.Assets, asset_id) < (qty - 2*Setup.CALC_TOLERANCE):
-            logging.fatal(g_tr('Ledger', "Conversion failed. Asset amount is not enogh. Date: ")
-                          + f"{datetime.fromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
+
+        processed_qty = 0
+        processed_value = 0
+        # Get asset amount accumulated before current operation
+        asset_amount = self.getAmount(BookAccount.Assets, asset_id)
+        if asset_amount < (qty - 2*Setup.CALC_TOLERANCE):
+            logging.fatal(g_tr('Ledger', "Asset amount is not enough for corporate action processing. Date: ")
+                          + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
             return
-        query = executeSQL(self.db,
-                           "SELECT d.open_sid AS open, ABS(coalesce(o.qty, ca.qty_new))- SUM(d.qty) AS remainder "
-                           "FROM deals AS d "
-                           "LEFT JOIN sequence AS os ON os.id=d.open_sid "
-                           "LEFT JOIN trades AS o ON o.id = os.operation_id AND os.type = 3 "
-                           "LEFT JOIN corp_actions AS ca ON ca.id = os.operation_id AND os.type = 5 "
-                           "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
-                           "GROUP BY d.open_sid "
-                           "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
+        # Get information about last deal
+        last_sid = readSQL(self.db, "SELECT "
+                                    "CASE WHEN qty>0 THEN open_sid ELSE close_sid END AS last_sid "
+                                    "FROM deals "
+                                    "WHERE account_id=:account_id AND asset_id=:asset_id "
+                                    "ORDER BY close_sid DESC, open_sid DESC LIMIT 1",
                            [(":account_id", account_id), (":asset_id", asset_id)])
-        last_sid = 0  # Take all Buy operations by default
-        reminder = 0
-        if query.next():  # Perform match ("closure") of previous Buy trades
-            # Get sid of Buy trade from the last deal and non-matched reminder of last Buy trade
-            last_sid, reminder = readSQLrecord(query)
-            # if last Buy is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
-            if reminder > 0:
-                last_sid = last_sid - 1
+        last_sid = 0 if last_sid is None else last_sid
+        # Next get information about abs trade quantity that was in this last deal
+        last_qty = readSQL(self.db, "SELECT coalesce(ABS(t.qty), 0)+coalesce(ABS(ca.qty_new) , 0) AS qty "
+                                    "FROM sequence AS s "
+                                    "LEFT JOIN trades AS t ON t.id=s.operation_id AND s.type=3 "
+                                    "LEFT JOIN corp_actions AS ca ON ca.id=s.operation_id AND s.type=5 "
+                                    "WHERE s.id=:last_sid", [(":last_sid", last_sid)])
+        last_qty = 0 if last_qty is None else last_qty
+        # Collect quantity of all deals where this last opposite trade participated (positive value)
+        deals_qty = readSQL(self.db, "SELECT coalesce(SUM(ABS(qty)), 0) "
+                                     "FROM deals AS d "
+                                     "WHERE account_id=:account_id AND asset_id=:asset_id "
+                                     "AND (open_sid=:last_sid OR close_sid=:last_sid)",
+                            [(":account_id", account_id), (":asset_id", asset_id), (":last_sid", last_sid)])
+        reminder = last_qty - deals_qty
+        # if last trade is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
+        if reminder > 0:
+            last_sid = last_sid - 1
+
+        # Get a list of all previous not matched trades or corporate actions of opposite direction (type parameter)
         query = executeSQL(self.db,
                            "SELECT * FROM ("
-                           "SELECT s.id, t.qty, t.price FROM trades AS t "
+                           "SELECT s.id, ABS(t.qty), t.price FROM trades AS t "
                            "LEFT JOIN sequence AS s ON s.type = 3 AND s.operation_id=t.id "
-                           "WHERE t.qty > 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
+                           "WHERE qty > 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
                            "AND s.id < :sid AND s.id > :last_sid "
                            "UNION ALL "
-                           "SELECT s.id, c.qty_new, coalesce(l.value/c.qty_new, 0) AS price FROM corp_actions AS c "
+                           "SELECT s.id, ABS(c.qty_new), coalesce(l.value/c.qty_new, 0) AS price FROM corp_actions AS c "
                            "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
                            "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id_new AND l.value > 0 "
-                           "WHERE c.qty_new > 0 AND c.asset_id_new = :asset_id AND c.account_id = :account_id "
+                           "WHERE c.qty_new > 0 AND c.asset_id_new=:asset_id AND c.account_id=:account_id "
                            "AND s.id < :sid AND s.id > :last_sid "
                            ")ORDER BY id",
                            [(":asset_id", asset_id), (":account_id", account_id),
                             (":sid", seq_id), (":last_sid", last_sid)])
-        while query.next():
+        while query.next():  # Perform match ("closure") of previous trades
             deal_sid, deal_qty, deal_price = readSQLrecord(query)  # deal_sid -> trade_sid
             if reminder > 0:
                 next_deal_qty = reminder
                 reminder = 0
             else:
                 next_deal_qty = deal_qty
-            if (buy_qty + next_deal_qty) >= qty:  # we are selling less or the same amount as was bought previously
-                next_deal_qty = qty - buy_qty
+            if (processed_qty + next_deal_qty) >= qty:  # We can't process more than qty of current trade
+                next_deal_qty = qty - processed_qty  # If it happens - just process the remainder of the trade
+            # Create a deal with relevant sign of quantity (-1 for short, +1 for long)
             _ = executeSQL(self.db, "INSERT INTO deals(account_id, asset_id, open_sid, close_sid, qty) "
                                     "VALUES(:account_id, :asset_id, :open_sid, :close_sid, :qty)",
                            [(":account_id", account_id), (":asset_id", asset_id), (":open_sid", deal_sid),
                             (":close_sid", seq_id), (":qty", next_deal_qty)])
-            buy_qty = buy_qty + next_deal_qty
-            buy_value = buy_value + (next_deal_qty * deal_price)
-            if buy_qty == qty:
+            processed_qty += next_deal_qty
+            processed_value += (next_deal_qty * deal_price)
+            if processed_qty == qty:
                 break
-        # Withdraw value with old quantity of old asset before conversion
-        self.appendTransaction(BookAccount.Assets, -buy_qty, -buy_value)
-        # Create the same value with new quantity of new asset after conversion
-        self.current['asset'] = self.current['peer']
-        self.appendTransaction(BookAccount.Assets, self.current['price'], buy_value)
+        # Asset allocations for different corporate actions:
+        # +-----------------+-------+-----+------------+-----------+----------+---------------+--------------------+
+        # |                 | Asset | Qty | cost basis | Asset new | Qty new  | cost basis    | The same algo for: |
+        # +-----------------+-------+-----+------------+-----------+----------+---------------+--------------------+
+        # | Symbol Change   |   A   |  N  |  100 %     |     B     |    N     |   100%        |                    |
+        # | (R-)Split       |   A   |  N  |  100 %     |     A     |    M     |   100%        | Stock Dividend     |
+        # | Merger          |   A   |  N  |  100 %     |     B     |    M     |   100%        |                    |
+        # | Spin-Off        |   A   |  N  |  100 %     |   A & B   | AxN, BxM | X% & (100-X)% |                    |
+        # +-----------------+-------+-----+------------+-----------+----------+---------------+--------------------+
+        # Withdraw value with old quantity of old asset as it common for all corporate action
+        self.appendTransaction(BookAccount.Assets, -processed_qty, -processed_value)
 
-    # Spin-Off is equal to Buy operation with 0 price
-    def processAssetEmission(self):
-        operation_details = self.current
-        # Values for TIMESTAMP, ACCOUNT_ID remains the same
-        self.current['asset'] = operation_details['peer']
-        self.current['amount'] = operation_details['price']
-        self.current['price'] = 0
-        self.current['category'] = 0
-        self.current['coupon'] = 0
-        self.current['peer'] = None
-        self.current['fee_tax'] = 0
-        self.current['tag'] = 0
-        self.processTrade()
-
-    def processCorporateAction(self):
-        operationCorpAction = {
-            CorporateAction.Merger: self.processAssetConversion,
-            CorporateAction.Split: self.processAssetConversion,
-            CorporateAction.SymbolChange: self.processAssetConversion,
-            CorporateAction.SpinOff: self.processAssetEmission,
-            CorporateAction.StockDividend: self.processAssetEmission
-        }
-        operationCorpAction[self.current['subtype']]()
+        # Prepare details about new asset
+        new_asset = self.current['peer']
+        new_qty = self.current['price']
+        new_value = processed_value
+        if self.current['subtype'] == CorporateAction.SpinOff:
+            new_value = processed_value * self.current['fee_tax']
+            # Modify value for old asset
+            self.appendTransaction(BookAccount.Assets, self.current['amount'], processed_value - new_value)
+        # Create value for new asset
+        self.current['asset'] = new_asset
+        self.appendTransaction(BookAccount.Assets, new_qty, new_value)
 
     # Rebuild transaction sequence and recalculate all amounts
     # timestamp:
@@ -509,7 +557,7 @@ class Ledger:
                     return
         if not silent:
             logging.info(g_tr('Ledger', "Re-build ledger from: ") +
-                         f"{datetime.fromtimestamp(frontier).strftime('%d/%m/%Y %H:%M:%S')}")
+                         f"{datetime.utcfromtimestamp(frontier).strftime('%d/%m/%Y %H:%M:%S')}")
         start_time = datetime.now()
         _ = executeSQL(self.db, "DELETE FROM deals WHERE close_sid >= "
                                 "(SELECT coalesce(MIN(id), 0) FROM sequence WHERE timestamp >= :frontier)",
@@ -535,14 +583,13 @@ class Ledger:
             if not silent and (query.at() % 1000) == 0:
                 logging.info(g_tr('Ledger', "Processed ") + f"{int(query.at()/1000)}" +
                              g_tr('Ledger', "k records, current frontier: ") +
-                             f"{datetime.fromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
+                             f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
         if fast_and_dirty:
             _ = executeSQL(self.db, "PRAGMA synchronous = ON")
 
-        end_time = datetime.now()
         if not silent:
-            logging.info(g_tr('Ledger', "Ledger is complete. Elapsed time: ") + f"{end_time - start_time}" +
-                         g_tr('Ledger', ", new frontier: ") + f"{datetime.fromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
+            logging.info(g_tr('Ledger', "Ledger is complete. Elapsed time: ") + f"{datetime.now() - start_time}" +
+                         g_tr('Ledger', ", new frontier: ") + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
         self.updateBalancesView()
         self.updateHoldingsView()
 
