@@ -329,7 +329,7 @@ class OperationsTimestampDelegate(QStyledItemDelegate):
         QStyledItemDelegate.__init__(self, parent)
 
     def displayText(self, value, locale):
-        return datetime.fromtimestamp(value).strftime('%d/%m/%Y %H:%M:%S')
+        return datetime.utcfromtimestamp(value).strftime('%d/%m/%Y %H:%M:%S')
 
     def paint(self, painter, option, index):
         painter.save()
@@ -338,7 +338,7 @@ class OperationsTimestampDelegate(QStyledItemDelegate):
         timestamp = record.value(index.column())
         transaction_type = record.value("type")
         number = record.value("num_peer")
-        text = datetime.fromtimestamp(timestamp).strftime('%d/%m/%Y %H:%M:%S')
+        text = datetime.utcfromtimestamp(timestamp).strftime('%d/%m/%Y %H:%M:%S')
         if (transaction_type == TransactionType.Trade) \
                 or (transaction_type == TransactionType.Dividend) \
                 or (transaction_type == TransactionType.CorporateAction):
@@ -435,9 +435,13 @@ class OperationsNotesDelegate(QStyledItemDelegate):
             symbol_after = record.value("note")
             qty_before = record.value("amount")
             qty_after = record.value("qty_trid")
-            text = CorpActionNames[sub_type].format(old=symbol_before, new=symbol_after,
-                                                         before=qty_before, after=qty_after) \
-                   + "\n" + record.value("note2")
+            basis = 100.0 * record.value("price")
+            if sub_type == CorporateAction.StockDividend:
+                qty_after = qty_after - qty_before
+            text = CorpActionNames[sub_type].format(old=symbol_before, new=symbol_after, before=qty_before,
+                                                    after=qty_after)
+            if sub_type == CorporateAction.SpinOff:
+                text += f"; {basis:.2f}% " + g_tr('OperationsDelegate', " cost basis") + "\n" + record.value("note2")
         else:
             assert False
         painter.drawText(option.rect, Qt.AlignLeft | Qt.AlignVCenter, text)
@@ -503,6 +507,8 @@ class OperationsAmountDelegate(QStyledItemDelegate):
             sub_type = record.value("fee_tax")
             qty_before = -record.value("amount")
             qty_after = record.value("qty_trid")
+            if sub_type == CorporateAction.StockDividend:
+                qty_after = qty_after + qty_before  # qty_before have been taken with '-' sign
             if sub_type == CorporateAction.SpinOff or sub_type == CorporateAction.StockDividend:
                 text = ""
             else:
@@ -543,7 +549,7 @@ class OperationsTotalsDelegate(QStyledItemDelegate):
             upper_part = f"{total_money:,.2f}"
         if transaction_type == TransactionType.CorporateAction:
             sub_type = record.value("fee_tax")
-            qty_before = record.value("amount") if sub_type ==CorporateAction.SpinOff else 0
+            qty_before = record.value("amount") if sub_type == CorporateAction.SpinOff else 0
             qty_after = record.value("t_qty") if sub_type == CorporateAction.StockDividend else record.value("qty_trid")
             if sub_type == CorporateAction.StockDividend:
                 text = f"\n{qty_after:,.2f}" if qty_after != '' else '\n<void>'
@@ -687,9 +693,9 @@ class ReportsTimestampDelegate(QStyledItemDelegate):
 
     def displayText(self, value, locale):
         if isinstance(value, str):  # already SQL-preprocessed date
-            text = datetime.fromtimestamp(int(value)).strftime('%d/%m/%Y')
+            text = datetime.utcfromtimestamp(int(value)).strftime('%d/%m/%Y')
         else:
-            text = datetime.fromtimestamp(value).strftime('%d/%m/%Y %H:%M:%S')
+            text = datetime.utcfromtimestamp(value).strftime('%d/%m/%Y %H:%M:%S')
         return text
 
 
@@ -698,7 +704,7 @@ class ReportsYearMonthDelegate(QStyledItemDelegate):
         QStyledItemDelegate.__init__(self, parent)
 
     def displayText(self, value, locale):
-        text = datetime.fromtimestamp(value).strftime('%Y %B')
+        text = datetime.utcfromtimestamp(value).strftime('%Y %B')
         return text
 
 class ReportsPandasDelegate(QStyledItemDelegate):
@@ -767,5 +773,5 @@ class SlipLinesPandasDelegate(QStyledItemDelegate):
         if index.column() == 1:
             model.setData(index, editor.selected_id)
             model.setData(index.siblingAtColumn(2), 1) # set confidence level to 1
-        if index.column() ==3:
+        if index.column() == 3:
             model.setData(index, editor.selected_id)
\ No newline at end of file
